/**
 * Patch: Reorder feature codes in the call_center group
 * Date: 2025-02-13
 * 
 * Purpose: This patch reorders the feature_code_id values for all feature codes in the 'call_center' group
 * to be ordered numerically by their default number (after stripping '*' and '#' characters).
 * All call_center feature codes will be placed after all non-call_center feature codes.
 * 
 * This version maintains referential integrity throughout the process without disabling foreign key checks.
 */

USE `ombutel`;

-- Start a transaction to ensure all operations complete together or not at all
START TRANSACTION;

-- Step 1: Create temporary tables to store feature code data and mapping
-- Create a table to hold all call center feature code data
CREATE TEMPORARY TABLE temp_call_center_feature_codes AS
SELECT fc.*
FROM `ombu_feature_codes` fc
JOIN `ombu_feature_code_groups` fcg USING (feature_code_group_id)
WHERE fcg.group_name = 'call_center'
ORDER BY CAST(REPLACE(REPLACE(fc.defaultnumber, '*', ''), '#', '') AS UNSIGNED);

-- Create a table to hold all call center feature code category members
CREATE TEMPORARY TABLE temp_call_center_members AS
SELECT fcm.*
FROM `ombu_feature_code_category_members` fcm
JOIN temp_call_center_feature_codes tcfc ON fcm.feature_code_id = tcfc.feature_code_id;

-- Create a table to store the mapping between old and new IDs
CREATE TEMPORARY TABLE temp_feature_code_mapping (
    old_id INT UNSIGNED NOT NULL,
    new_id INT UNSIGNED NOT NULL,
    PRIMARY KEY (old_id)
);

-- Step 2: Generate new IDs starting after the highest ID of non-call_center feature codes
INSERT INTO temp_feature_code_mapping (old_id, new_id)
SELECT 
    tccfc.feature_code_id AS old_id,
    (@rank := @rank + 1) + (
        SELECT COALESCE(MAX(feature_code_id), 0)
        FROM `ombu_feature_codes` fc2
        LEFT JOIN `ombu_feature_code_groups` fcg2 USING (feature_code_group_id)
        WHERE fcg2.group_name != 'call_center'
    ) AS new_id
FROM 
    (SELECT @rank := 0) r,
    temp_call_center_feature_codes tccfc
ORDER BY 
    CAST(REPLACE(REPLACE(tccfc.defaultnumber, '*', ''), '#', '') AS UNSIGNED);

-- Step 3: Delete all existing call center feature codes and their category memberships
-- (We have already saved them in the temporary tables)
DELETE FROM `ombu_feature_code_category_members`
WHERE feature_code_id IN (SELECT feature_code_id FROM temp_call_center_feature_codes);

DELETE FROM `ombu_feature_codes`
WHERE feature_code_id IN (SELECT feature_code_id FROM temp_call_center_feature_codes);

-- Step 4: Re-insert the call center feature codes with their new IDs
INSERT INTO `ombu_feature_codes`
SELECT 
    tm.new_id AS feature_code_id,
    tccfc.feature_code_group_id,
    tccfc.feature_name,
    tccfc.description,
    tccfc.defaultnumber,
    tccfc.customnumber,
    tccfc.func,
    tccfc.exten,
    tccfc.prefix,
    tccfc.sufix,
    tccfc.sendlength,
    tccfc.param01,
    tccfc.param02,
    tccfc.param03,
    tccfc.param04,
    tccfc.param05,
    tccfc.state,
    tccfc.edit,
    tccfc.generate,
    tccfc.display
FROM 
    temp_call_center_feature_codes tccfc
    JOIN temp_feature_code_mapping tm ON tccfc.feature_code_id = tm.old_id;

-- Step 5: Re-insert the category members with the new feature_code_ids
INSERT INTO `ombu_feature_code_category_members` (feature_code_category_id, feature_code_id)
SELECT 
    tccm.feature_code_category_id,
    tm.new_id
FROM 
    temp_call_center_members tccm
    JOIN temp_feature_code_mapping tm ON tccm.feature_code_id = tm.old_id;

-- Step 6: Clean up - drop temporary tables
DROP TEMPORARY TABLE IF EXISTS temp_call_center_feature_codes;
DROP TEMPORARY TABLE IF EXISTS temp_call_center_members;
DROP TEMPORARY TABLE IF EXISTS temp_feature_code_mapping;

-- Commit the transaction
COMMIT;

-- Verification query (commented out, uncomment to check results)
-- SELECT fc.feature_code_id, fc.defaultnumber, fcg.group_name
-- FROM `ombu_feature_codes` fc
-- JOIN `ombu_feature_code_groups` fcg USING (feature_code_group_id)
-- WHERE fcg.group_name = 'call_center'
-- ORDER BY fc.feature_code_id;
